#include "SceneOpenGL.h"


// Permet d'viter la r-criture du namespace glm::

using namespace glm;


// Constructeur de Destucteur

SceneOpenGL::SceneOpenGL(std::string titreFenetre, int largeurFenetre, int hauteurFenetre) : m_titreFenetre(titreFenetre), m_largeurFenetre(largeurFenetre),
                                                                                             m_hauteurFenetre(hauteurFenetre), m_fenetre(0), m_contexteOpenGL(0),
                                                                                             m_input()
{

}


SceneOpenGL::~SceneOpenGL()
{
    SDL_GL_DeleteContext(m_contexteOpenGL);
    SDL_DestroyWindow(m_fenetre);
    SDL_Quit();
}


// Mthodes

bool SceneOpenGL::initialiserFenetre()
{
    // Initialisation de la SDL

    if(SDL_Init(SDL_INIT_VIDEO) < 0)
    {
        std::cout << "Erreur lors de l'initialisation de la SDL : " << SDL_GetError() << std::endl;
        SDL_Quit();

        return false;
    }


    #ifdef __APPLE__

        // Version d'OpenGL

        SDL_GL_SetAttribute(SDL_GL_CONTEXT_MAJOR_VERSION, 3);
        SDL_GL_SetAttribute(SDL_GL_CONTEXT_MINOR_VERSION, 2);
        SDL_GL_SetAttribute(SDL_GL_CONTEXT_PROFILE_MASK, SDL_GL_CONTEXT_PROFILE_CORE);


        // Rcupration du Bundle

        CFURLRef URLBundle = CFBundleCopyResourcesDirectoryURL(CFBundleGetMainBundle());
        char *cheminResources = new char[PATH_MAX];


        // Changement du 'Working Directory'

        if(CFURLGetFileSystemRepresentation(URLBundle, 1, (UInt8*)cheminResources, PATH_MAX))
            chdir(cheminResources);


        // Libration de la mmoire

        delete[] cheminResources;
        CFRelease(URLBundle);

    #else

        // Version d'OpenGL

        SDL_GL_SetAttribute(SDL_GL_CONTEXT_MAJOR_VERSION, 3);
        SDL_GL_SetAttribute(SDL_GL_CONTEXT_MINOR_VERSION, 3);
        SDL_GL_SetAttribute(SDL_GL_CONTEXT_PROFILE_MASK, SDL_GL_CONTEXT_PROFILE_CORE);

    #endif


    // Cration de la fentre

    m_fenetre = SDL_CreateWindow(m_titreFenetre.c_str(), SDL_WINDOWPOS_CENTERED, SDL_WINDOWPOS_CENTERED, m_largeurFenetre, m_hauteurFenetre, SDL_WINDOW_SHOWN | SDL_WINDOW_OPENGL);

    if(m_fenetre == 0)
    {
        std::cout << "Erreur lors de la creation de la fenetre : " << SDL_GetError() << std::endl;
        SDL_Quit();

        return false;
    }


    // Cration du contexte OpenGL

    m_contexteOpenGL = SDL_GL_CreateContext(m_fenetre);

    if(m_contexteOpenGL == 0)
    {
        std::cout << SDL_GetError() << std::endl;
        SDL_DestroyWindow(m_fenetre);
        SDL_Quit();

        return false;
    }

    return true;
}


bool SceneOpenGL::initGL()
{
    #ifdef WIN32

        // On initialise GLEW

        GLenum initialisationGLEW( glewInit() );


        // Si l'initialisation a chou :

        if(initialisationGLEW != GLEW_OK)
        {
            // On affiche l'erreur grce  la fonction : glewGetErrorString(GLenum code)

            std::cout << "Erreur d'initialisation de GLEW : " << glewGetErrorString(initialisationGLEW) << std::endl;


            // On quitte la SDL

            SDL_GL_DeleteContext(m_contexteOpenGL);
            SDL_DestroyWindow(m_fenetre);
            SDL_Quit();

            return false;
        }

    #endif


    // Activation du Depth Buffer

    glEnable(GL_DEPTH_TEST);


    // Tout s'est bien pass, on retourne true

    return true;
}


void SceneOpenGL::bouclePrincipale()
{
    // Variables

    unsigned int frameRate (1000 / 50);
    Uint32 debutBoucle(0), finBoucle(0), tempsEcoule(0);


    // Frame Buffer

    FrameBuffer frameBuffer(512, 512);
    frameBuffer.charger();


    // Matrices (premire passe)

    mat4 projectionFBO;
    mat4 modelviewFBO;

    projectionFBO = perspective(70.0, (double)frameBuffer.getLargeur() / frameBuffer.getHauteur(), 1.0, 100.0);
    modelviewFBO = mat4(1.0);


    // Matrices (seconde passe)

    mat4 projection;
    mat4 modelview;

    projection = perspective(70.0, (double) m_largeurFenetre / m_hauteurFenetre, 1.0, 100.0);
    modelview = mat4(1.0);


    // Camra mobile

    Camera camera(vec3(3, 3, 3), vec3(0, 0, 0), vec3(0, 1, 0), 0.5, 0.5);
    m_input.afficherPointeur(false);
    m_input.capturerPointeur(true);


    // Objet Caisse

    Caisse caisse(2.0, "Shaders/textureMVP.vert", "Shaders/texture.frag", "Textures/Caisse2.jpg");
    caisse.charger();

    float angle = 0.0;


    // Vertices

    float vertices[] = {0, 0, 0,   4, 0, 0,   4, 4, 0,     // Triangle 1
    			0, 0, 0,   0, 4, 0,   4, 4, 0};    // Triangle 2

    float coordTexture[] = {0, 0,   1, 0,   1, 1,          // Triangle 1
			    0, 0,   0, 1,   1, 1};         // Triangle 2


    /* ***** Gestion du VBO ***** */

    GLuint vbo;
    int tailleVerticesBytes = 18 * sizeof(float);
    int tailleCoordTextureBytes = 12 * sizeof(float);


    // Gnration du VBO

    glGenBuffers(1, &vbo);


    // Verrouillage

    glBindBuffer(GL_ARRAY_BUFFER, vbo);

	// Remplissage

	glBufferData(GL_ARRAY_BUFFER, tailleVerticesBytes + tailleCoordTextureBytes, 0, GL_STATIC_DRAW);
	glBufferSubData(GL_ARRAY_BUFFER, 0, tailleVerticesBytes, vertices);
	glBufferSubData(GL_ARRAY_BUFFER, tailleVerticesBytes, tailleCoordTextureBytes, coordTexture);


    // Dverrouillage

    glBindBuffer(GL_ARRAY_BUFFER, 0);


    /* ***** Gestion du VAO ***** */

    GLuint vao;


    // Gnration du VAO

    glGenVertexArrays(1, &vao);


    // Verrouillage du VAO

    glBindVertexArray(vao);

	// Verrouillage du VBO

	glBindBuffer(GL_ARRAY_BUFFER, vbo);


		// Vertex Attrib 0 (Vertices)

		glVertexAttribPointer(0, 3, GL_FLOAT, GL_FALSE, 0, BUFFER_OFFSET(0));
		glEnableVertexAttribArray(0);


		 // Vertex Attrib 0 (Vertices)

		glVertexAttribPointer(2, 2, GL_FLOAT, GL_FALSE, 0, BUFFER_OFFSET(tailleVerticesBytes));
		glEnableVertexAttribArray(2);


	// Dverrouillage du VBO

	glBindBuffer(GL_ARRAY_BUFFER, 0);


    // Dverrouillage du VAO

    glBindVertexArray(0);


    // Shader

    Shader shaderTexture("Shaders/textureMVP.vert", "Shaders/texture.frag");
    shaderTexture.charger();


    // Boucle principale

    while(!m_input.terminer())
    {
        // On dfinit le temps de dbut de boucle

        debutBoucle = SDL_GetTicks();


        // Gestion des vnements

        m_input.updateEvenements();

        if(m_input.getTouche(SDL_SCANCODE_ESCAPE))
            break;

        camera.deplacer(m_input);



        /* ***** Premire Passe ***** */


        // Verrouillage du Frame Buffer

        glBindFramebuffer(GL_FRAMEBUFFER, frameBuffer.getID());


            // Nettoyage de l'cran

            glClearColor(0.5, 0.5, 0.5, 1.0);
            glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);


            // Redimensionnement de la zone d'affichage

            glViewport(0, 0, frameBuffer.getLargeur(), frameBuffer.getHauteur());


            // Placement de la camra

            modelviewFBO = lookAt(vec3(3, 0, 3), vec3(0, 0, 0), vec3(0, 1, 0));


            // Gestion de la rotation de la caisse

            angle += 2;

            if(angle > 360)
                angle -= 360;


            // Affichage de la caisse

            mat4 sauvegardeModelviewFBO = modelviewFBO;

                modelviewFBO = rotate(modelviewFBO, angle, vec3(0, 1, 0));
                caisse.afficher(projectionFBO, modelviewFBO);

            modelviewFBO = sauvegardeModelviewFBO;


        // Dverrouillage du Frame Buffer

        glBindFramebuffer(GL_FRAMEBUFFER, 0);



        /* ***** Seconde Passe ***** */


        // Nettoyage de l'cran

        glClearColor(0.0, 0.0, 0.0, 1.0);
        glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);


        // Redimensionnement de la zone d'affichage

        glViewport(0, 0, m_largeurFenetre, m_hauteurFenetre);


        // Gestion de la camra

        camera.lookAt(modelview);


        // Activation du shader

        glUseProgram(shaderTexture.getProgramID());


        // Verrouillage du VAO

        glBindVertexArray(vao);


            // Envoi des matrices

                shaderTexture.envoyerMat4("modelviewProjection", projection * modelview);


            // Verrouillage de la texture

            glBindTexture(GL_TEXTURE_2D, frameBuffer.getColorBufferID(0));


            // Rendu

            glDrawArrays(GL_TRIANGLES, 0, 6);


            // Dverrouillage de la texture

                glBindTexture(GL_TEXTURE_2D, 0);


            // Verrouillage du VAO

            glBindVertexArray(0);


        // Dsactivation du shader

        glUseProgram(0);


        // Actualisation de la fentre

        SDL_GL_SwapWindow(m_fenetre);


        // Calcul du temps coul

        finBoucle = SDL_GetTicks();
        tempsEcoule = finBoucle - debutBoucle;


        // Si ncessaire, on met en pause le programme

        if(tempsEcoule < frameRate)
            SDL_Delay(frameRate - tempsEcoule);
    }
}
